#ifndef GENPROC_H
#define GENPROC_H

#include "stdinc.h"

#include <QtOpenGL/QGLWidget>

// structure to store a point (distance from it in distance field)
struct Point { 
	float x,y; // point coordinate
	float color[3]; // color of all points in dinstance field, nearer to THIS point as to any other point or line
};

// line structure (same as above)
struct Line {
	float p[2], r[2]; // line coefficients: g(t) = p + t * r;
	float color[3];  // color of all points in dinstance field, nearer to THIS line as to any other point or line
};

typedef std::vector<Point> PointType;
typedef std::vector<Line> LineType;


class GenProc
{
public:
	GenProc();
	~GenProc();
	bool loadFile(const char *filename); // load textfile with geometry description
	void loadShader(); // load shader from file, compile and link them to programs, get variable locations
	void performComputation(void); // the core function, it activates the dinstance calculation in the fragment program
	void paintResult(const GLint windowWidth, const GLint windowHeight); // draw results onto screen
	void printResults(char *filename); // print computation results to file
	void resetDistanceField();
	bool isFileLoaded() { return fileLoaded; }

private:
	void reinit(); // initialization stuff and computation of the distance field
	void initDistanceField(); // initialize distance fielt for upload to graphic memory (texture)

	void initFBO(void); // initialization of framebuffer object
	void createTextures(void);  // create and setup textures (read and write)
	void setupTexture( const GLuint texID ); // needed by createTextures()
	void transferToTexture( float* data, GLuint texID ); // "upload" data to texture ( Mainmem -> GPU-MEM)
	void transferFromTexture(float* data); // "download" data from texture (GPU-MEM -> Mainmem)
	void initGLSL(void); // initialize GLSL - shader programs
	void computationStep(); // help function for performComputation()

	void cleanShaderPrograms();
	void cleanTextures();

	void swapTextures(void); // switch indices to adress id's for read/write Texture
	void checkGLErrors( const char *label ); // check and print gl errors to stderr
	bool checkFramebufferStatus(void); // framebufferstatus to stdout

	PointType points; // vector storing points, from which distances are calculated
	LineType lines;// vector storing lines, from which distances are calculated

	bool npot_supported; // non power of two textures supportet (or not)
	unsigned int fieldWidth; // width of the distance field
	unsigned int fieldHeight; // height of the distance field
	float *distanceField; // distance field, this is needed to initialize texture on GPU-Memory

	GLenum texInternalFormat; // internal format of textures, representing the distance field
	GLuint fbo; // variable to store framebuffer object id

	// GLSL vars
	GLuint sh_prog_id_point; // id of shader program to calculate dinstance field to a point
	GLuint sh_prog_id_line; // id of shader program to calculate dinstance field to a line
	GLuint sh_prog_id_visualize; // id of shader program to transfer distance field into a color texture
	GLint texPointUni; // texture location in fragment program distfield_point.frag
	GLint texLineUni; // texture location in fragment program distfield_line.frag
	GLint texVisUni; // texture location in fragment program in visualizeDF.frag
	GLint pointUni; // point coordinate location in fragment program
	GLint lineUni; // line coordinate location in fragment program
	GLint pointColorUni; // point coordinate location in fragment program
	GLint lineColorUni; // line coordinate location in fragment program
	GLint texSizeRcpUni;
	

	// ping pong management vars
	unsigned int writeTex; // index to address id's of read/write textures
	unsigned int readTex; //....
	GLenum attachmentpoints[2]; // where to attack framebufferobjects
	GLuint texIDs[2]; // texture ids of read/write Textures

	bool fileLoaded;
};

#endif